#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests that test the functionality of the capability version REST API.
The logic can be found in `capability/views/capability_version.py
"""
import http

import pytest
from pyramid.testing import DummyRequest

# pylint: disable=C0415, E0401


def test_view_latest_version(request_null_capability: DummyRequest):
    from capability.views.capability_version import view_latest_version

    request_null_capability.matchdict["capability_request_id"] = 1
    response = view_latest_version(request_null_capability)
    expected_json = {
        "type": "CapabilityVersion",
        "capability_request_id": 1,
        "version_number": 1,
        "state": None,
        "sealed": None,
        "parameters": '{"arguments": "-pe"}',
        "workflow_metadata": None,
        "files": [],
        "current_execution": {
            "capability_request_id": 1,
            "created_at": "2022-01-04T19:40:59.963133",
            "current_workflow_request_id": None,
            "delivery_url": None,
            "id": 0,
            "queue_state": None,
            "state_name": None,
            "type": "CapabilityExecution",
            "updated_at": "2022-01-04T19:40:59.963144",
            "version_number": None,
        },
    }
    assert response.status_code == http.HTTPStatus.OK

    actual_json = response.json_body
    for key in expected_json.keys():
        if key != "current_execution":
            assert actual_json[key] == expected_json[key]
        else:
            expd_curr_exec = expected_json["current_execution"]
            actual_curr_exec = actual_json["current_execution"]
            for ce_key, val in actual_curr_exec.items():
                if ce_key not in ("current_execution", "created_at", "updated_at"):
                    assert val == expd_curr_exec[ce_key]

    request_null_capability.matchdict["capability_request_id"] = 2
    no_versions_response = view_latest_version(request_null_capability)
    assert no_versions_response.status_code == http.HTTPStatus.PRECONDITION_FAILED

    request_null_capability.matchdict["capability_request_id"] = 6
    no_such_request_response = view_latest_version(request_null_capability)
    assert no_such_request_response.status_code == http.HTTPStatus.NOT_FOUND

    # we -should- find this one
    expected_json = {
        "type": "CapabilityVersion",
        "capability_request_id": 21,
        "version_number": 2,
        "state": None,
        "sealed": None,
        "parameters": {
            "product_locator": "uid://evla/execblock/35b23178-0dde-4167-82fd-9bd6128a80a1",
            "cal_locator": "uid://evla/calibration/6a940d35-c714-47a2-ac4a-1df6182ee83f",
            "user_email": "jgoldste@nrao.edu",
        },
        "workflow_metadata": {"carta_url": "https://pr-dsoc-dev.nrao.edu/pKP8pcGvI02M/"},
        "files": [],
        "current_execution": None,
    }
    request_null_capability.matchdict["capability_request_id"] = expected_json["capability_request_id"]
    request_null_capability.matchdict["capability_name"] = "std_restore_imaging"
    not_found_response = view_latest_version(request_null_capability)
    assert not_found_response.status_code == http.HTTPStatus.OK
    assert not_found_response.json_body == expected_json


def test_create_capability_version(request_null_capability: DummyRequest):
    """
    Tests the create capability version view to make sure it properly supports creation of versions

    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability_version import create_capability_version

    request_null_capability.matchdict["capability_request_id"] = 1
    request_null_capability.json_body = {"parameters": "-g"}

    # Assert test capability version not in list of versions (mocked)
    request = request_null_capability.capability_info.lookup_capability_request(1)
    assert len(request.versions) != 3

    response = create_capability_version(request_null_capability)
    assert response.status_code == http.HTTPStatus.OK
    expected_response = {
        "type": "CapabilityVersion",
        "capability_request_id": 1,
        "version_number": 2,
        "parameters": "-g",
    }
    for entry in expected_response:
        assert entry in response.json_body
    # Assert test capability version has been added to list of versions (mocked)
    request = request_null_capability.capability_info.lookup_capability_request(1)
    assert request.versions[1]
